/**
 * Copyright (c) 2009 - 2018, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
* @brief Example template project.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include <stdint.h>

#include "nrf.h"
#include "nordic_common.h"
#include "boards.h"

#include "nrf_gpio.h"
#include "nrf_delay.h"

#define RAM_MEMORY_TEST_ADDRESS (0x20008000UL) 
#define RAM_MEMORY_TEST_WORD    (0xFEEDBEEFUL)

// uint32_t * volatile p_ram_test = (uint32_t *)RAM_MEMORY_TEST_ADDRESS;
uint32_t ram_value __attribute__((section(".retained_section"))) = 0;
/**
 * @brief Function for application main entry.
 */
int main(void)
{
    // Configure LEDs as outputs and turn them off(active low)
    nrf_gpio_cfg_output(17);
    nrf_gpio_pin_set(17);
    
    nrf_gpio_cfg_output(18);
    nrf_gpio_pin_set(18);
    
    // Check if chip is waking up after a SoftReset.
    if(NRF_POWER->GPREGRET == 0xBB)
    {  
        //if(*(uint32_t *)p_ram_test == RAM_MEMORY_TEST_WORD){
        if(ram_value == RAM_MEMORY_TEST_WORD){
            // Turn on LED1 if *p_ram_test match the RAM_MEMORY_TEST_WORD after SoftReset
            nrf_gpio_pin_clear(17);
        }
        else
        {
            // Turn on LED2 if *p_ram_test does not matche the RAM_MEMORY_TEST_WORD after SoftReset
             nrf_gpio_pin_clear(18);
        }
        
        // Reset GPREGRET register
        NRF_POWER->GPREGRET = 0x00;
        
        // Loop until a manual pin reset is performed by pressing the RESET button on the nRF52 DK
        while (true)
        {
            // Do nothing.
        }
    }
    
    // Store magic number in GPREGRET register to indicate that the p_ram_test conten should be checked upon reset. 
    NRF_POWER->GPREGRET = 0xBB;
    
    // Write test word to RAM
    //*p_ram_test = RAM_MEMORY_TEST_WORD;
    ram_value = RAM_MEMORY_TEST_WORD;
    // Configure the RAM retention parameters
    NRF_POWER->RAMON |= (POWER_RAMON_OFFRAM0_RAM0On << POWER_RAMON_OFFRAM0_Pos) 
                        | (POWER_RAMON_OFFRAM1_RAM1On << POWER_RAMON_OFFRAM1_Pos);
    
    // Delay for one second 
    nrf_delay_ms(1000);
    
    //Perform SoftReset
    NVIC_SystemReset();
    
    // Should not end up here, but just in case; hang tight.
    while (true)
    {
        // Do nothing.
    }
}
/** @} */
